-- To create a Schemabound view on every table use
-- the procedure sp_CreateSchemaboundViewToPreventTableDrop
-- For example, USE pubs and then 
-- EXEC sp_CreateSchemaboundViewToPreventTableDrop 'ALL'
-- to create a schemabound view on every table.
-- EXEC sp_depends authors
-- then re-check the dependencies!

USE master
GO

/* ***********************************************************
** NOTE - This does NOT work IF any of the columns of your
** primary key have a comma followed by a space OR brackets 
** (neither Left nor Right [ or ]) in their column names. 
**
** Also, if your table DOES NOT have a primary key then the 
** first column defined for the table cannot have these things 
** either.
*********************************************************** */

IF OBJECTPROPERTY(object_id('sp_GetPrimaryKeyList'), 
	'IsProcedure') = 1
	DROP PROCEDURE sp_GetPrimaryKeyList
GO

CREATE PROCEDURE sp_GetPrimaryKeyList
(
	@TableName		sysname = NULL,
	@ColumnKeyList	nvarchar(2100) = NULL OUTPUT
)
AS
SET NOCOUNT ON
IF @TableName IS NULL
BEGIN
	RAISERROR('You must supply a valid tablename in order to return the Primary Key Index List.', 16, 1)
	RETURN
END

IF (OBJECTPROPERTY(object_id(@TableName), 'IsUserTable') IS NULL OR
		OBJECTPROPERTY(object_id(@TableName), 'IsUserTable') = 0)
BEGIN
	RAISERROR('Table:%s is not a valid TABLE name.', 16, 1, @TableName)
	RETURN
END

CREATE TABLE #IndexInfo
(
	IndexName	sysname,
	IndexDesc	varchar(210),
	IndexKeys	nvarchar(2126)
)

INSERT INTO #IndexInfo 
	EXEC sp_helpindex @TableName

SELECT @ColumnKeyList = IndexKeys
	FROM #IndexInfo 
		WHERE IndexDesc LIKE '%primary key%'

SELECT @ColumnKeyList = '[' + replace(@ColumnKeyList, ', ', '], [') + ']'
GO

IF OBJECTPROPERTY(object_id('sp_GetPrimaryKeyList'), 'ISProcedure') = 1
  BEGIN
	PRINT 'Added procedure dbo.sp_GetPrimaryKeyList to master database.'
  END
GO

-- Test the sp_GetPrimaryKeyList procedure
-- USE pubs
-- DECLARE @ColumnKeyList	nvarchar(2100)
-- EXEC sp_GetPrimaryKeyList sales, @ColumnKeyList	OUTPUT
-- SELECT @ColumnKeyList

IF OBJECTPROPERTY(object_id('dbo.sp_GenerateUniqueNameForNewObject'), 
	'IsProcedure') = 1
	DROP PROCEDURE dbo.sp_GenerateUniqueNameForNewObject
GO

CREATE PROCEDURE [dbo].sp_GenerateUniqueNameForNewObject
			@TableName	sysname,
			@NewObjName	sysname OUTPUT,
			@Qualifier	nvarchar(50),
			@PreORPost	bit = 1
AS
IF @PreORPost IS NULL	-- This is to handle a case where someone EXPLICITLY passes NULL
	SET @PreORPost = 1	-- 1 which is the default is to POSTFIX the identifier

DECLARE	@Counter 	int,
	@RetStatValid	bit

SELECT @Counter = 1

-- Start with a simple postfixed name 
-- making sure the new name does not get beyond 128 characters.
IF @PreORPost = 1
	SELECT @NewObjName = substring(@TableName, 1, 
							128-LEN(@Qualifier)) + @Qualifier
IF @PreORPost = 0
	SELECT @NewObjName = @Qualifier + substring(@TableName, 1, 
							128-LEN(@Qualifier)) 

IF NOT EXISTS (SELECT * FROM dbo.sysobjects WHERE [name] = @NewObjName)
  BEGIN
	-- DO NOT QUOTE IT UNTIL EXECUTION! ALWAYS QUOTENAME IN OBJECTPROPERTY FUNCTIONS
	-- SELECT @NewObjName = QUOTENAME(@NewObjName, ']')
	RETURN
  END
ELSE -- If the simple pre/postfixed name doesn't work - uniqify it!
  BEGIN
	WHILE EXISTS (SELECT * FROM dbo.sysobjects WHERE [name] = @NewObjName)
	  BEGIN
		IF @PreORPost = 1
		  BEGIN
			SELECT @NewObjName = 
					substring(@TableName, 1, 
						128-LEN(@Qualifier)
							-LEN(convert(varchar(10), @Counter)))
						+ @Qualifier + convert(varchar(10), @Counter)
		  END
		IF @PreORPost = 0
		  BEGIN
			SELECT @NewObjName = 
					@Qualifier 
						+ substring(@TableName, 1, 
							128-LEN(@Qualifier)
								-LEN(convert(varchar(10), @Counter)))
						+ convert(varchar(10), @Counter)
		  END
		IF NOT EXISTS(SELECT * FROM sysobjects WHERE name = @NewObjName)
		  BEGIN
			RETURN
		  END
		SELECT @Counter = @Counter + 1	
	  END
  END
go

IF OBJECTPROPERTY(object_id('dbo.sp_GenerateUniqueNameForNewObject'), 'ISProcedure') = 1
  BEGIN
	PRINT 'Added procedure dbo.sp_GenerateUniqueNameForNewObject to master database.'
  END
go

-- Test the procedure sp_GenerateUniqueNameForNewObject
-- USE pubs
-- DECLARE @NewObjName	sysname
-- EXEC sp_GenerateUniqueNameForNewObject
-- 	@TableName 	= 'this is a test to see if a large table name will work - even if it has spaces and [brackets] in the name it should work just fine!',
-- 	@NewObjName = @NewObjName OUTPUT,
-- 	@Qualifier 	= '_PreventTableDropView'
-- SELECT @NewObjName 

IF OBJECTPROPERTY(object_id('sp_CreateSchemaboundViewToPreventTableDrop'), 
	'IsProcedure') = 1
	DROP PROCEDURE sp_CreateSchemaboundViewToPreventTableDrop
GO

CREATE PROCEDURE sp_CreateSchemaboundViewToPreventTableDrop
	@TableName	sysname = NULL
AS
SET NOCOUNT ON
IF @TableName IS NULL
BEGIN
	RAISERROR('You must supply a valid tablename or ALL to process all tables.', 16, 1)
	RETURN
END

IF @TableName <> 'ALL' AND (OBJECTPROPERTY(object_id(@TableName), 'IsUserTable') IS NULL OR
		OBJECTPROPERTY(object_id(@TableName), 'IsUserTable') = 0)
BEGIN
	RAISERROR('Table:%s is not a valid TABLE name. You must supply a valid tablename or use ALL to process all user-defined tables.', 16, 1, @TableName)
	RETURN
END

DECLARE @CurrentTableName	nvarchar(260),
		@PrintStr			nvarchar(1000),
		@CrViewExecStr		nvarchar(4000),
		@ColumnKeyList		nvarchar(2100),
		@SBVName			sysname

IF UPPER(@TableName) = 'ALL'
  BEGIN
	DECLARE TableNamesCursor CURSOR LOCAL FAST_FORWARD FOR 
		SELECT [Name] from dbo.sysobjects
	         WHERE OBJECTPROPERTY(id, 'IsUserTable') = 1 
				AND STATUS > 0 AND 
				NOT([Name] = 'dtproperties')
				ORDER BY [Name]
	OPEN TableNamesCursor
    FETCH NEXT FROM TableNamesCursor INTO @CurrentTableName
    WHILE (@@fetch_status <> -1)
      BEGIN
		EXEC sp_GenerateUniqueNameForNewObject
				@Tablename 	= @CurrentTableName,
				@NewObjName	= @SBVName OUTPUT,
				@Qualifier	= '_PreventTableDropWithSBView'
		SELECT @CurrentTableName = QUOTENAME(@CurrentTableName, ']'),
				@SBVName		 = QUOTENAME(@SBVName, ']')
		SELECT @PrintStr = 'Creating Schemabound View named ' + @SBVName + ' for ' + 
			RTRIM(UPPER(@CurrentTableName))
		PRINT @PrintStr 
		IF OBJECTPROPERTY(object_id(@CurrentTableName), 'TableHasPrimaryKey') = 1
		  BEGIN
			EXEC sp_GetPrimaryKeyList @CurrentTableName, @ColumnKeyList OUTPUT
			SELECT @PrintStr = '    Using the Primary Key ON ' 
				+ @ColumnKeyList + ' for ' + RTRIM(UPPER(@CurrentTableName))
			PRINT @PrintStr 
		  END
		ELSE
		  BEGIN
			SELECT @PrintStr = '    Table ' + RTRIM(UPPER(@CurrentTableName)) +
				' does not have a Primary Key, using the first column defined instead.'
			PRINT @PrintStr 
			SELECT @ColumnKeyList = QUOTENAME([Name], ']')
				FROM syscolumns 
					WHERE id = object_id(@CurrentTableName)
						AND colid = (SELECT min(colid) 
									FROM syscolumns 
									WHERE id = object_id(@CurrentTableName))
		  END
		SELECT @CrViewExecStr = 'CREATE VIEW dbo.' + @SBVName
					+ ' WITH SCHEMABINDING AS SELECT ' 
					+ @ColumnKeyList 
					+ ' FROM dbo.' + @CurrentTableName
		-- SELECT @CrViewExecStr AS ViewDefintion
		EXEC (@CrViewExecStr)
		PRINT ' '
	    FETCH NEXT FROM TableNamesCursor INTO @CurrentTableName
	  END
  END
DEALLOCATE TableNamesCursor
GO

IF OBJECTPROPERTY(object_id('dbo.sp_CreateSchemaboundViewToPreventTableDrop'), 'ISProcedure') = 1
  BEGIN
	PRINT 'Added procedure dbo.sp_CreateSchemaboundViewToPreventTableDrop to master database.'
  END
go

-- Test the procedure sp_CreateSchemaboundViewToPreventTableDrop
-- USE pubs
-- EXEC sp_CreateSchemaboundViewToPreventTableDrop 'ALL'
-- EXEC sp_depends authors

IF OBJECTPROPERTY(object_id('dbo.sp_DropAllSBPreventTableDropViews'), 
	'IsProcedure') = 1
	DROP PROCEDURE dbo.sp_DropAllSBPreventTableDropViews
GO

CREATE PROCEDURE dbo.sp_DropAllSBPreventTableDropViews
	@ViewName	sysname = NULL
AS
SET NOCOUNT ON
IF @ViewName IS NULL
BEGIN
	RAISERROR('You must supply a valid viewname or ALL to drop all views with Postfix "_PreventTableDropWithSBView".', 16, 1)
	RETURN
END

IF UPPER(@ViewName) <> 'ALL' AND 
	(@ViewName NOT LIKE '%_PreventTableDropWithSBView%'
	OR OBJECTPROPERTY(object_id(@ViewName), 'IsView') IS NULL 
	OR OBJECTPROPERTY(object_id(@ViewName), 'IsView') = 0)
BEGIN
	RAISERROR('View:%s is not a valid View name. To drop these views you must supply individual viewnames that end with postfix "_PreventTableDropWithSBView". Use ALL to remove all views with Postfix "_PreventTableDropWithSBView".', 16, 1, @ViewName)
	RETURN
END

DECLARE @CurrentTableName	nvarchar(260),
		@PrintStr			nvarchar(1000),
		@DrViewExecStr		nvarchar(4000)

IF UPPER(@ViewName) <> 'ALL'
  BEGIN
	SELECT @DrViewExecStr = 'DROP VIEW ' + QUOTENAME(@ViewName, ']')
	EXEC (@DrViewExecStr)
  END

ELSE
  BEGIN
	DECLARE ViewNamesCursor CURSOR LOCAL FAST_FORWARD FOR 
		SELECT [Name] FROM dbo.sysobjects
	         WHERE Type = 'V' AND Name LIKE '%_PreventTableDropWithSBView%'
				ORDER BY [Name]
	OPEN ViewNamesCursor
    FETCH NEXT FROM ViewNamesCursor INTO @ViewName
    WHILE (@@fetch_status <> -1)
      BEGIN
		SELECT @DrViewExecStr = 'DROP VIEW ' + QUOTENAME(@ViewName, ']')
		EXEC (@DrViewExecStr)
	    FETCH NEXT FROM ViewNamesCursor INTO @ViewName
	  END
  END
DEALLOCATE ViewNamesCursor
GO

IF OBJECTPROPERTY(object_id('dbo.sp_DropAllSBPreventTableDropViews'), 'ISProcedure') = 1
  BEGIN
	PRINT 'Added procedure dbo.sp_DropAllSBPreventTableDropViews to master database.'
  END
go

-- Test the procedure dbo.sp_DropAllSBPreventTableDropViews
-- USE pubstest
-- EXEC sp_depends authors
-- EXEC sp_DropAllSBPreventTableDropViews 'ALL'
-- EXEC sp_depends authors